package javaCodeGen;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import CodeGen.IModuleGen;
import MDL.*;

public class ModuleJavaGen extends JavaElement implements IModuleGen{
	private Module m;

	public ModuleJavaGen(Module module) {
		m = module;
	}
	
	@Override
	public String genCode(String filePath) {
		String code = null;
		try{ 
	    	FileWriter fstream = new FileWriter(filePath); //Create file
	    	BufferedWriter out = new BufferedWriter(fstream);
	    	code = genCode(); //generate the code
	    	out.write(code); //write to the file
	    	out.close();	//Close the output stream
	    	fstream.write(code);
	    	fstream.close();
		}catch (Exception e){
			System.err.println("Error: " + e.getMessage());
		}
		return code;
	}
	@Override
	public String genCode(){
		setI(0);
		addL("package modules;\n");		
		genComment();
		addL("public class " + m.getName() + "{");
		genGetName();
		genMethodCode();		
		addL("}");			
		return getResult();
	}
	
	@Override
	public void genGetName() {
		setI(1);
		addL("/**");
		addL(" * this could be useful when working in the next layer in the packet");
		addL(" * @return the module's name");		 
		addL(" */");
		addL("public String getModuleName(){");
		addL("return \"" + m.getName() + "\";", 1);
		addL("}");
		setI();
	}

	@Override
	public void genComment() {
		setI(0);
		addL("/*");
		addL(" *" + m.getComment());
		addL(" */");
		setI();
	}

	@Override
	public void genMethodCode() {
		setI(1);
		Iterator<Method> methods = m.getMethods();
		MethodJavaGen method;
		while(methods.hasNext()){			
			method = new MethodJavaGen(methods.next());
			addL();
			method.genCommentForConsts();
			method.genComponentsArray();				
			method.genConstantArray();
			method.genPacketLen();		
			addL();
			method.genCreatorComment();
			method.genCreatorCode();
			method.genCheckPacketType();
			method.genParserCode();	
		}
		setI();
	}
}
